

set -ex



pip check
jupyter server extension list
jupyter server extension list 2>&1 | grep -ie "jupyterlab_git.*OK"
jupyter labextension list
jupyter labextension list 2>&1 | grep -ie "@jupyterlab/git.*OK.*(python, jupyterlab-git)"
exit 0
