"use strict";(self.webpackChunk_jupyterlab_git=self.webpackChunk_jupyterlab_git||[]).push([[122],{2122:(t,e,r)=>{function n(){return n=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var n in r)Object.prototype.hasOwnProperty.call(r,n)&&(t[n]=r[n])}return t},n.apply(this,arguments)}r.d(e,{Z:()=>n})}}]);